#
# $Id: testsuite.rb,v 1.4 2001/08/06 05:08:40 akira Exp $
#
# Copyright (c) 2001 akira yamada <akira@ruby-lang.org>
# You can redistribute it and/or modify it under the same term as Ruby.
#

require 'runit/testcase'
require 'runit/testsuite'
require 'runit/cui/testrunner'

require 'uri'

class TestURI < RUNIT::TestCase
  def setup
    super
    @url = 'http://a/b/c/d;p?q'
    @base_url = URI.parse(@url)
  end

  def test_parse
    # 0
    assert_kind_of(URI::HTTP, @base_url)

    exp = [
      'http', 
      nil, 'a', URI::HTTP.default_port, 
      '/b/c/d;p', 
      'q',
      nil
    ]
    ary = @base_url.to_ary
    assert_equal(exp, ary)

    # 1
    url = URI.parse('ftp://ftp.is.co.za/rfc/rfc1808.txt')
    assert_kind_of(URI::FTP, url)

    exp = [
      'ftp', 
      nil, 'ftp.is.co.za', URI::FTP.default_port, 
      '/rfc/rfc1808.txt', nil,
    ]
    ary = url.to_ary
    assert_equal(exp, ary)

    # 2
    url = URI.parse('gopher://spinaltap.micro.umn.edu/00/Weather/California/Los%20Angeles')
    assert_kind_of(URI::Generic, url)

    exp = [
      'gopher', 
      nil, 'spinaltap.micro.umn.edu', nil, nil,
      '/00/Weather/California/Los%20Angeles', nil,
      nil,
      nil
    ]
    ary = url.to_ary
    assert_equal(exp, ary)

    # 3
    url = URI.parse('http://www.math.uio.no/faq/compression-faq/part1.html')
    assert_kind_of(URI::HTTP, url)

    exp = [
      'http', 
      nil, 'www.math.uio.no', URI::HTTP.default_port, 
      '/faq/compression-faq/part1.html', 
      nil,
      nil
    ]
    ary = url.to_ary
    assert_equal(exp, ary)

    # 4
    url = URI.parse('mailto:mduerst@ifi.unizh.ch')
    assert_kind_of(URI::Generic, url)

    exp = [
      'mailto', 
      'mduerst@ifi.unizh.ch',
      []
    ]
    ary = url.to_ary
    assert_equal(exp, ary)

    # 5
    url = URI.parse('news:comp.infosystems.www.servers.unix')
    assert_kind_of(URI::Generic, url)

    exp = [
      'news', 
      nil, nil, nil, nil, 
      nil, 'comp.infosystems.www.servers.unix',
      nil,
      nil
    ]
    ary = url.to_ary
    assert_equal(exp, ary)

    # 6
    url = URI.parse('telnet://melvyl.ucop.edu/')
    assert_kind_of(URI::Generic, url)

    exp = [
      'telnet', 
      nil, 'melvyl.ucop.edu', nil, nil, 
      '/', nil,
      nil,
      nil
    ]
    ary = url.to_ary
    assert_equal(exp, ary)
  end

#  http://a/b/c/d;p?q
#        g:h           =  g:h
  def test_merge_01
    url = @base_url.merge('g:h')
    assert_kind_of(URI::Generic, url)
    assert_equal('g:h', url.to_s)
  end
  def test_route_01
    url = @base_url.route_to('g:h')
    assert_kind_of(URI::Generic, url)
    assert_equal('g:h', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g             =  http://a/b/c/g
  def test_merge_02
    url = @base_url.merge('g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g', url.to_s)
  end
  def test_route_02
    url = @base_url.route_to('http://a/b/c/g')
    assert_kind_of(URI::Generic, url)
    assert_equal('g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ./g           =  http://a/b/c/g
  def test_merge_03
    url = @base_url.merge('./g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g', url.to_s)
  end
  def test_route_02
    url = @base_url.route_to('http://a/b/c/g')
    assert_kind_of(URI::Generic, url)
    assert('./g' != url.to_s) # ok
    assert_equal('g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g/            =  http://a/b/c/g/
  def test_merge_04
    url = @base_url.merge('g/')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g/', url.to_s)
  end
  def test_route_04
    url = @base_url.route_to('http://a/b/c/g/')
    assert_kind_of(URI::Generic, url)
    assert_equal('g/', url.to_s)
  end

#  http://a/b/c/d;p?q
#        /g            =  http://a/g
  def test_merge_05
    url = @base_url.merge('/g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/g', url.to_s)
  end
  def test_route_05
    url = @base_url.route_to('http://a/g')
    assert_kind_of(URI::Generic, url)
    assert('/g' != url.to_s) # ok
    assert_equal('../../g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        //g           =  http://g
  def test_merge_06
    url = @base_url.merge('//g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://g', url.to_s)
  end
  def test_route_06
    url = @base_url.route_to('http://g')
    assert_kind_of(URI::Generic, url)
    assert_equal('//g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ?y            =  http://a/b/c/?y
  def test_merge_07
    url = @base_url.merge('?y')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/?y', url.to_s)
  end
  def test_route_07
    url = @base_url.route_to('http://a/b/c/?y')
    assert_kind_of(URI::Generic, url)
    assert_equal('?y', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g?y           =  http://a/b/c/g?y
  def test_merge_08
    url = @base_url.merge('g?y')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g?y', url.to_s)
  end
  def test_route_08
    url = @base_url.route_to('http://a/b/c/g?y')
    assert_kind_of(URI::Generic, url)
    assert_equal('g?y', url.to_s)
  end

#  http://a/b/c/d;p?q
#        #s            =  (current document)#s
  def test_merge_09
    url = @base_url.merge('#s')
    assert_kind_of(URI::HTTP, url)
    assert_equal(@base_url.to_s + '#s', url.to_s)
  end
  def test_route_09
    url = @base_url.route_to(@base_url.to_s + '#s')
    assert_kind_of(URI::Generic, url)
    assert_equal('#s', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g#s           =  http://a/b/c/g#s
  def test_merge_10
    url = @base_url.merge('g#s')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g#s', url.to_s)
  end
  def test_route_10
    url = @base_url.route_to('http://a/b/c/g#s')
    assert_kind_of(URI::Generic, url)
    assert_equal('g#s', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g?y#s         =  http://a/b/c/g?y#s
  def test_merge_11
    url = @base_url.merge('g?y#s')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g?y#s', url.to_s)
  end
  def test_route_11
    url = @base_url.route_to('http://a/b/c/g?y#s')
    assert_kind_of(URI::Generic, url)
    assert_equal('g?y#s', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ;x            =  http://a/b/c/;x
  def test_merge_12
    url = @base_url.merge(';x')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/;x', url.to_s)
  end
  def test_route_12
    url = @base_url.route_to('http://a/b/c/;x')
    assert_kind_of(URI::Generic, url)
    assert_equal(';x', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g;x           =  http://a/b/c/g;x
  def test_merge_13
    url = @base_url.merge('g;x')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g;x', url.to_s)
  end
  def test_route_13
    url = @base_url.route_to('http://a/b/c/g;x')
    assert_kind_of(URI::Generic, url)
    assert_equal('g;x', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g;x?y#s       =  http://a/b/c/g;x?y#s
  def test_merge_14
    url = @base_url.merge('g;x?y#s')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g;x?y#s', url.to_s)
  end
  def test_route_14
    url = @base_url.route_to('http://a/b/c/g;x?y#s')
    assert_kind_of(URI::Generic, url)
    assert_equal('g;x?y#s', url.to_s)
  end

#  http://a/b/c/d;p?q
#        .             =  http://a/b/c/
  def test_merge_15
    url = @base_url.merge('.')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/', url.to_s)
  end
  def test_route_15
    url = @base_url.route_to('http://a/b/c/')
    assert_kind_of(URI::Generic, url)
    assert('.' != url.to_s) # ok
    assert_equal('./', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ./            =  http://a/b/c/
  def test_merge_16
    url = @base_url.merge('./')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/', url.to_s)
  end
  def test_route_16
    url = @base_url.route_to('http://a/b/c/')
    assert_kind_of(URI::Generic, url)
    assert_equal('./', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ..            =  http://a/b/
  def test_merge_17
    url = @base_url.merge('..')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/', url.to_s)
  end
  def test_route_17
    url = @base_url.route_to('http://a/b/')
    assert_kind_of(URI::Generic, url)
    assert('..' != url.to_s) # ok
    assert_equal('../', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ../           =  http://a/b/
  def test_merge_18
    url = @base_url.merge('../')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/', url.to_s)
  end
  def test_route_18
    url = @base_url.route_to('http://a/b/')
    assert_kind_of(URI::Generic, url)
    assert_equal('../', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ../g          =  http://a/b/g
  def test_merge_19
    url = @base_url.merge('../g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/g', url.to_s)
  end
  def test_route_19
    url = @base_url.route_to('http://a/b/g')
    assert_kind_of(URI::Generic, url)
    assert_equal('../g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ../..         =  http://a/
  def test_merge_20
    url = @base_url.merge('../..')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/', url.to_s)
  end
  def test_route_20
    url = @base_url.route_to('http://a/')
    assert_kind_of(URI::Generic, url)
    assert('../..' != url.to_s) # ok
    assert_equal('../../', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ../../        =  http://a/
  def test_merge_21
    url = @base_url.merge('../../')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/', url.to_s)
  end
  def test_route_21
    url = @base_url.route_to('http://a/')
    assert_kind_of(URI::Generic, url)
    assert_equal('../../', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ../../g       =  http://a/g
  def test_merge_22
    url = @base_url.merge('../../g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/g', url.to_s)
  end
  def test_route_22
    url = @base_url.route_to('http://a/g')
    assert_kind_of(URI::Generic, url)
    assert_equal('../../g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        <>            =  (current document)
  def test_merge_23
    url = @base_url.merge('')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/d;p?q', url.to_s)
  end
  def test_route_23
    url = @base_url.route_to('http://a/b/c/d;p?q')
    assert_kind_of(URI::Generic, url)
    assert_equal('', url.to_s)
  end

#  http://a/b/c/d;p?q
#        /./g          =  http://a/./g
  def test_merge_24
    url = @base_url.merge('/./g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/./g', url.to_s)
  end
  def test_route_24
    url = @base_url.route_to('http://a/./g')
    assert_kind_of(URI::Generic, url)
    assert_equal('/./g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        /../g         =  http://a/../g
  def test_merge_25
    url = @base_url.merge('/../g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/../g', url.to_s)
  end
  def test_route_25
    url = @base_url.route_to('http://a/../g')
    assert_kind_of(URI::Generic, url)
    assert_equal('/../g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g.            =  http://a/b/c/g.
  def test_merge_26
    url = @base_url.merge('g.')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g.', url.to_s)
  end
  def test_route_26
    url = @base_url.route_to('http://a/b/c/g.')
    assert_kind_of(URI::Generic, url)
    assert_equal('g.', url.to_s)
  end

#  http://a/b/c/d;p?q
#        .g            =  http://a/b/c/.g
  def test_merge_27
    url = @base_url.merge('.g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/.g', url.to_s)
  end
  def test_route_27
    url = @base_url.route_to('http://a/b/c/.g')
    assert_kind_of(URI::Generic, url)
    assert_equal('.g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g..           =  http://a/b/c/g..
  def test_merge_28
    url = @base_url.merge('g..')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g..', url.to_s)
  end
  def test_route_28
    url = @base_url.route_to('http://a/b/c/g..')
    assert_kind_of(URI::Generic, url)
    assert_equal('g..', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ..g           =  http://a/b/c/..g
  def test_merge_29
    url = @base_url.merge('..g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/..g', url.to_s)
  end
  def test_route_29
    url = @base_url.route_to('http://a/b/c/..g')
    assert_kind_of(URI::Generic, url)
    assert_equal('..g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ../../../g    =  http://a/../g
  def test_merge_30
    url = @base_url.merge('../../../g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/../g', url.to_s)
  end
  def test_route_30
    url = @base_url.route_to('http://a/../g')
    assert_kind_of(URI::Generic, url)
    assert('../../../g' != url.to_s) # ok? yes, it confuses you
    assert_equal('/../g', url.to_s)  # and it is clearly
  end

#  http://a/b/c/d;p?q
#        ../../../../g =  http://a/../../g
  def test_merge_31
    url = @base_url.merge('../../../../g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/../../g', url.to_s)
  end
  def test_route_30
    url = @base_url.route_to('http://a/../../g')
    assert_kind_of(URI::Generic, url)
    assert('../../../../g' != url.to_s) # ok? yes, it confuses you
    assert_equal('/../../g', url.to_s)  # and it is clearly
  end

#  http://a/b/c/d;p?q
#        ./../g        =  http://a/b/g
  def test_merge_32
    url = @base_url.merge('./../g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/g', url.to_s)
  end
  def test_route_32
    url = @base_url.route_to('http://a/b/g')
    assert_kind_of(URI::Generic, url)
    assert('./../g' != url.to_s) # ok
    assert_equal('../g', url.to_s)
  end

#  http://a/b/c/d;p?q
#        ./g/.         =  http://a/b/c/g/
  def test_merge_33
    url = @base_url.merge('./g/.')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g/', url.to_s)
  end
  def test_route_33
    url = @base_url.route_to('http://a/b/c/g/')
    assert_kind_of(URI::Generic, url)
    assert('./g/.' != url.to_s) # ok
    assert_equal('g/', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g/./h         =  http://a/b/c/g/h
  def test_merge_34
    url = @base_url.merge('g/./h')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g/h', url.to_s)
  end
  def test_route_34
    url = @base_url.route_to('http://a/b/c/g/h')
    assert_kind_of(URI::Generic, url)
    assert('g/./h' != url.to_s) # ok
    assert_equal('g/h', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g/../h        =  http://a/b/c/h
  def test_merge_35
    url = @base_url.merge('g/../h')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/h', url.to_s)
  end
  def test_route_35
    url = @base_url.route_to('http://a/b/c/h')
    assert_kind_of(URI::Generic, url)
    assert('g/../h' != url.to_s) # ok
    assert_equal('h', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g;x=1/./y     =  http://a/b/c/g;x=1/y
  def test_merge_36
    url = @base_url.merge('g;x=1/./y')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g;x=1/y', url.to_s)
  end
  def test_route_36
    url = @base_url.route_to('http://a/b/c/g;x=1/y')
    assert_kind_of(URI::Generic, url)
    assert('g;x=1/./y' != url.to_s) # ok
    assert_equal('g;x=1/y', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g;x=1/../y    =  http://a/b/c/y
  def test_merge_37
    url = @base_url.merge('g;x=1/../y')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/y', url.to_s)
  end
  def test_route_37
    url = @base_url.route_to('http://a/b/c/y')
    assert_kind_of(URI::Generic, url)
    assert('g;x=1/../y' != url.to_s) # ok
    assert_equal('y', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g?y/./x       =  http://a/b/c/g?y/./x
  def test_merge_38
    url = @base_url.merge('g?y/./x')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g?y/./x', url.to_s)
  end
  def test_route_38
    url = @base_url.route_to('http://a/b/c/g?y/./x')
    assert_kind_of(URI::Generic, url)
    assert_equal('g?y/./x', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g?y/../x      =  http://a/b/c/g?y/../x
  def test_merge_39
    url = @base_url.merge('g?y/../x')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g?y/../x', url.to_s)
  end
  def test_route_39
    url = @base_url.route_to('http://a/b/c/g?y/../x')
    assert_kind_of(URI::Generic, url)
    assert_equal('g?y/../x', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g#s/./x       =  http://a/b/c/g#s/./x
  def test_merge_40
    url = @base_url.merge('g#s/./x')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g#s/./x', url.to_s)
  end
  def test_route_40
    url = @base_url.route_to('http://a/b/c/g#s/./x')
    assert_kind_of(URI::Generic, url)
    assert_equal('g#s/./x', url.to_s)
  end

#  http://a/b/c/d;p?q
#        g#s/../x      =  http://a/b/c/g#s/../x
  def test_merge_41
    url = @base_url.merge('g#s/../x')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://a/b/c/g#s/../x', url.to_s)
  end
  def test_route_41
    url = @base_url.route_to('http://a/b/c/g#s/../x')
    assert_kind_of(URI::Generic, url)
    assert_equal('g#s/../x', url.to_s)
  end

#  http://a/b/c/d;p?q
#        http:g        =  http:g           ; for validating parsers
#                      |  http://a/b/c/g   ; for backwards compatibility
  def test_merge_42
    url = @base_url.merge('http:g')
    assert_kind_of(URI::HTTP, url)
    assert_equal('http:g', url.to_s)
  end
  def test_route_42
    url = @base_url.route_to('http:g')
    assert_kind_of(URI::Generic, url)
    assert_equal('http:g', url.to_s)
  end

# from [ruby-dev:11508] Re: uri
  def test_merge_X01
    url = URI.parse('http://hoge/a.html') + 'b.html'
    assert_kind_of(URI::HTTP, url)
    assert_equal('http://hoge/b.html', url.to_s)
  end
  def test_route_X01
    url = URI.parse('http://hoge/a.html').route_to('http://hoge/b.html')
    assert_kind_of(URI::Generic, url)
    assert_equal('b.html', url.to_s)
  end

  def test_ftp_01
    url = URI.parse('ftp://user:pass@host.com/abc/def')
    assert_kind_of(URI::FTP, url)

    exp = [
      'ftp',
      'user:pass', 'host.com', URI::FTP.default_port, 
      '/abc/def', nil,
    ]
    ary = url.to_ary
    assert_equal(exp, ary)

    assert_equal('user', url.user)
    assert_equal('pass', url.password)
  end
end

if $0 == __FILE__
  if ARGV.size == 0
    suite = TestURI.suite
  else
    suite = RUNIT::TestSuite.new
    ARGV.each do |testmethod|
      suite.add_test(TestURI.new(testmethod))
    end
  end
  RUNIT::CUI::TestRunner.run(suite)
end
